import "Turbine.UI";
import "Turbine.UI.Lotro";
local importPath=string.gsub(getfenv(1)._.Name,"%.TabControl","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

if fontMetricInst==nil then
	import (importPath.."FontMetric");
	fontMetricInst=FontMetric();
	if settings==nil then
		settings={}
	end
	if settings.Font==nil then settings.Font=Turbine.UI.Lotro.Verdana16 end
	fontMetricInst:SetFont(settings.Font);
end

TabButton=class(Turbine.UI.Control)
function TabButton:Constructor()
	Turbine.UI.Control.Constructor(self);
	Turbine.UI.Control.SetSize(self,100,22); -- default size
	self.LeftImage=Turbine.UI.Control();
	self.LeftImage:SetParent(self);
	self.LeftImage:SetSize(4,22);
	self.LeftImage:SetPosition(0,0)
	self.LeftImage:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.LeftImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
	self.Label=Turbine.UI.Label();
	self.Label:SetParent(self);
	self.Label:SetSize(92,22);
	self.Label:SetPosition(4,0);
	self.Label:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.Label:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
	self.Label:SetTextAlignment(Turbine.UI.ContentAlignment.TopCenter);
	self.Label:SetMultiline(false);
	self.RightImage=Turbine.UI.Control();
	self.RightImage:SetParent(self);
	self.RightImage:SetSize(4,22);
	self.RightImage:SetPosition(96,0)
	self.RightImage:SetBlendMode(Turbine.UI.BlendMode.Overlay);
	self.RightImage:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
	self.LeftImage:SetMouseVisible(false);
	self.Label:SetMouseVisible(false);
	self.RightImage:SetMouseVisible(false);

	self.SetUpImages=function(sender,imageLeft,imageMiddle,imageRight)
		self.LeftImageUp=imageLeft;
		self.MiddleImageUp=imageMiddle;
		self.RightImageUp=imageRight;
	end
	self.SetDownImages=function(sender,imageLeft,imageMiddle,imageRight)
		self.LeftImageDown=imageLeft;
		self.MiddleImageDown=imageMiddle;
		self.RightImageDown=imageRight;
	end
	self.SetState=function(sender,state)
		state=tonumber(state)
		if state==nil then state=0 end
		if state==0 then
			self.LeftImage:SetBackground(self.LeftImageDown);
			self.Label:SetBackground(self.MiddleImageDown);
			self.RightImage:SetBackground(self.RightImageDown);
		else
			self.LeftImage:SetBackground(self.LeftImageUp);
			self.Label:SetBackground(self.MiddleImageUp);
			self.RightImage:SetBackground(self.RightImageUp);
		end
	end
	self.SetForeColor=function(sender,color)
		self.Label:SetForeColor(color);
	end
	self.SetBackColor=function(sender,color)
		self.LeftImage:SetBackColor(color);
		self.RightImage:SetBackColor(color);
		self.Label:SetBackColor(color);
	end
	self.SetBlendMode=function(sender,blendmode)
		self.LeftImage:SetBlendMode(blendmode);
		self.Label:SetBlendMode(blendmode);
		self.RightImage:SetBlendMode(blendmode);
	end
	self.SetBackColorBlendMode=function(sender,blendmode)
		self.LeftImage:SetBackColorBlendMode(blendmode);
		self.Label:SetBackColorBlendMode(blendmode);
		self.RightImage:SetBackColorBlendMode(blendmode);
	end
	self.SetText=function(sender,text)
		self.Label:SetText(text);
		local parent=self:GetParent();
		fontMetricInst:SetFont(Font);
		parent.ButtonWidth=9;
		for k,v in ipairs(parent.Tabs) do
			local TextWidth=fontMetricInst:GetTextWidth(v.Button:GetText(),20)+12; -- add 8 pixels for tab borders+4 pixels for text padding
			if TextWidth>parent.ButtonWidth then
				parent.ButtonWidth=TextWidth;
			end
		end
		parent:Layout();
	end
	self.GetText=function(sender)
		return self.Label:GetText();
	end
	self.SetFont=function(sender,font)
		self.Label:SetFont(font);
	end
	self.SetTextAlignment=function(sender,alignment)
		self.Label:SetTextAlignment(alignment)
	end
	self.SizeChanged=function()
		local width,height=self:GetSize();
		self.Label:SetWidth(width-8);
		self.RightImage:SetLeft(width-4);
	end
end

TabControl=class(Turbine.UI.Control)
function TabControl:Constructor()
	Turbine.UI.Control.Constructor(self);
	self.TabRows=1;
	self.ActiveTab=0;
	self.ButtonWidth=9;
	self.ButtonBack=Turbine.UI.Control();
	self.ButtonBack:SetParent(self);
	self.ButtonBack:SetSize(self:GetWidth(),22);
	self.ButtonBack:SetBackColor(Turbine.UI.Color.Black);
	self.TabBorder=Turbine.UI.Control();
	self.TabBorder:SetParent(self);
	self.TabBorder:SetSize(self:GetWidth(),self:GetHeight()-self.TabRows*22);
	self.TabBorder:SetPosition(0,self.TabRows*20);
	self.TabBorder:SetZOrder(1);
	self.Font=Turbine.UI.Lotro.Font.Verdana18;
	self.Tabs={}

	self.SetTab=function(sender,tmpIndex)
		for k,v in ipairs(self.Tabs) do
			if k==tmpIndex then
				self.TabBorder:SetBackColor(v.BorderColor);
				v.Button:SetState(1);
--				v.Button:SetTop(v.Button)
				v:SetVisible(true);
				if v.Activate ~= nil then
					v.Activate();
				end
			else
				if k==self.ActiveTab then
					v.Button:SetState(0);
					v:SetVisible(false);
					if v.Deactivate ~= nil then
						v.Deactivate();
					end
				end
			end
		end
		self.ActiveTab=tmpIndex;
		self:Layout()
	end

	self.AddTab=function(sender,newName,backColor,panel)
		if newName==nil then newName="Tab "..#self.Tabs+1 end
		if backColor==nil then backColor=Turbine.UI.Color.DarkGray end

		if panel==nil then
			panel=Turbine.UI.Control();
		end
		panel:SetParent(self.TabBorder);
		panel.BorderColor=backColor
		panel:SetBackColor(Turbine.UI.Color(1,backColor.R*.188,backColor.G*.188,backColor.B*.188))
		panel:SetVisible(false);
		panel:SetSize(self.TabBorder:GetWidth()-4,self.TabBorder:GetHeight()-4);
		panel:SetPosition(2,2);
		panel.Button=TabButton();
		panel.Button:SetParent(self);
		panel.Button:SetFont(self.Font);
		panel.Button:SetText(newName);
		panel.Button:SetDownImages(resourcePath.."TabDownLeft.tga",resourcePath.."TabDownMiddle.tga",resourcePath.."TabDownRight.tga")
		panel.Button:SetUpImages(resourcePath.."TabUpLeft.tga",resourcePath.."TabUpMiddle.tga",resourcePath.."TabUpRight.tga")
		panel.Button:SetState(0);
		panel.Button:SetBlendMode(Turbine.UI.BlendMode.Overlay);
		panel.Button:SetBackColor(backColor);
		panel.Button:SetBackColorBlendMode(Turbine.UI.BlendMode.Color);
		panel.Button.Index=#self.Tabs+1;

		local textWidth=fontMetricInst:GetTextWidth(newName,20)+12; -- add 8 pixels for tab borders+4 pixels for text padding
		if textWidth>self.ButtonWidth then
			self.ButtonWidth=textWidth
		end

		panel.Button.MouseClick=function(sender,args)
			self:SetTab(sender.Index)
		end
		panel.SetText=function(sender,Text)
			panel.Button:SetText(Text);
		end

		table.insert(self.Tabs,panel);
		if self.ActiveTab==0 then self:SetTab(1) end
		self:Layout();
		return panel;
	end
	self.SizeChanged=function(sender,args)
		if self:GetWidth()<self.ButtonWidth then
			self:SetWidth(self.ButtonWidth)
		else
			self:Layout();
		end
	end
	self.SetFont=function(sender,font)
		self.Font=font;
		-- resize all buttons
		fontMetricInst:SetFont(font);
		self.ButtonWidth=9;
		for k,v in ipairs(self.Tabs) do
			v.Button:SetFont(font);
			v.Button:SetText(v.Button:GetText());

			local textWidth=fontMetricInst:GetTextWidth(v.Button:GetText(),20)+12; -- add 8 pixels for tab borders+4 pixels for text padding
			if textWidth>self.ButtonWidth then
				self.ButtonWidth=textWidth;
			end
		end
		self:Layout();
	end
	self.Layout=function()
		local newWidth,newHeight=self:GetSize();
		local buttonsPerRow=math.floor(newWidth/self.ButtonWidth);
		self.TabRows=#self.Tabs/buttonsPerRow;
		if self.TabRows~=math.floor(self.TabRows) then self.TabRows=math.floor(self.TabRows)+1 end
		if self.ActiveTab>0 then
			self.ActiveRow=math.floor((self.ActiveTab-1)/buttonsPerRow)+1
		else
			self.ActiveRow=self.TabRows
		end
		self.TabBorder:SetSize(newWidth,newHeight-self.TabRows*20);
		self.TabBorder:SetTop(self.TabRows*20);
		self.ButtonBack:SetSize(newWidth,self.TabRows*20);
		for k,v in ipairs(self.Tabs) do
			local tmpRow=math.floor((v.Button.Index-1)/buttonsPerRow)+1
			if tmpRow==self.ActiveRow then
				if k==self.ActiveTab then
					v.Button:SetTop((self.TabRows-1)*20);
					v.Button:SetZOrder(self.TabRows+1);
				else
					v.Button:SetTop((self.TabRows-1)*20+2);
					v.Button:SetZOrder(self.TabRows-1);
				end
			else
				if tmpRow<self.ActiveRow then
					v.Button:SetTop((tmpRow-1)*20+2);
					v.Button:SetZOrder(tmpRow-1);
				else
					v.Button:SetTop((tmpRow-2)*20+2);
					v.Button:SetZOrder(tmpRow-2);
				end
			end
			v.Button:SetLeft((v.Button.Index-(tmpRow-1)*buttonsPerRow-1)*self.ButtonWidth);
			v.Button:SetWidth(self.ButtonWidth);
			v:SetSize(newWidth-4,self.TabBorder:GetHeight()-4)
		end
		self.TabBorder:SetZOrder(self.TabRows);
	end
	self.SetBackColor=function(sender,color)
		Turbine.UI.Control.SetBackColor(self,color);
		self.ButtonBack:SetBackColor(color);
	end
end